.include "m8def.inc"


.org 0x0000
        rjmp    main                    ;Reset Handler

.org OC2addr
           rjmp    timer2_compare   ; Timer Compare Handler

.org OVF0addr
        rjmp    timer0_overflow


.def temp01 = r16
.def temp02 = r17
.def moduleNumber = r18
.def moduleInstruction = r19
.def controlBits = r20
.def IrCommandNr = r21
.def irLEDcounter = r22
.def moduleIRid = r23
.def motorRun = r24
.def IRcommand = r25
.def lightOn01 = r26
.def lightOn02 = r27

.set MODULE_ID_TO_SEND = 15



main:
        nop                            ;Do nothing
        ldi     temp01, LOW(RAMEND)     ;initialize stackpointer
        out     SPL, temp01
        ldi     temp01, HIGH(RAMEND)
        out     SPH, temp01


        ;Initialize timer 0 (8-bit) - Time motor turns table

        ldi     temp01, ( 1 << CS02 ) | ( 1 << CS00 );Prescaler 1024
        out     TCCR0, temp01


        ;Initialize timer 2 (8-bit) - IR communication
                                    ;Compare value
        ldi     temp01, 5
        out     OCR2, temp01
                                    ;Switch to CTC mode, prescaler 256
                                    ;4 000 000/64/12 = 488Hz = 2.048ms
        ldi     temp01, ( 1 << WGM21 ) | ( 1 << CS22 )
        out     TCCR2, temp01

        ;Switch to interrupt mode of ALL timers
        ldi     temp01, ( 1 << OCIE2 ) | ( 1 <<  TOIE0);TOIE0 interrupt overflow timer 0, OCIE1A interrupt at Timer Compare timer 1, OCIE2 interrupt at timer 2
        out     TIMSK, temp01



        ;Initialize ports
        ldi     temp01, 0b00100001
        out     DDRB, temp01
        ldi     temp01, 0b00000000     ;No pullups at communication ports!
        out     PORTB, temp01

        ldi     temp01, 0b00111111
        out     DDRC, temp01
        ldi     temp01, 0b00000000
        out     PORTC, temp01

        ldi     temp01, 0b00001111
        out     DDRD, temp01
        ldi     temp01, 0b00000000
        out     PORTD, temp01

        sei


;++++++++++++++++++++++++
;Turn on light for a short time to indicate that microcontroller is initializing
        sbi     PORTC, 0
        sbi     PORTB, 0

        ldi     lightOn01, 100
        ldi     lightOn02, 0

initialize:
        cpi     lightOn01, 1
        brsh    initialize

        cbi     PORTC, 0
        cbi     PORTB, 0

;------------------------


        clr     controlBits
        clr     irLEDcounter
        clr     motorRun
        clr     moduleIRid
        clr     moduleInstruction
        clr     IRcommand
        clr     IRcommandNr
        clr     lightOn01
        clr     lightOn02

loop:

;Receive data from wired connection if PINB 3 is high
        sbis    PINB, 3
        rjmp    skipInput
        cli

        clr     temp01
        clr     temp02
        rcall   readInput
        cpi     temp02, 0b11111111
        breq    endInput
        mov     moduleNumber, temp01

        clr     temp01
        rcall   readInput
        cpi     temp02, 0b11111111
        breq    endInput
        mov     moduleInstruction, temp01

transmissionEnd:
        sbic    PINB, 3
        rjmp    transmissionEnd

;Process received instructions from wired connection
        cpi     moduleNumber, 1
        brne    skipInstruction
        cpi     moduleInstruction, 1
        brne    nextInstruction01
        sbi     PORTB, 0
nextInstruction01:
        cpi     moduleInstruction, 5
        brne    nextInstruction02
        cbi     PORTD, 0    ;Turn table clockwise
        sbi     PORTD, 1
        ldi     MotorRun, 100
nextInstruction02:
        cpi     moduleInstruction, 10
        brne    nextInstruction03
        sbi     PORTD, 0    ;Turn table anticlockwise
        cbi     PORTD, 1
        ldi     MotorRun, 100
nextInstruction03:
        cpi     moduleInstruction, 15
        brne    nextInstruction04
        sbi     PORTC, 0 ;Turn light on
        ldi     lightOn01, 255
        ldi     lightOn02, 10
nextInstruction04:
        cpi     moduleInstruction, 20
        brne    nextInstruction05
        cbi     PORTC, 0 ;Turn light off
nextInstruction05:

        clr     moduleInstruction

skipInstruction:

endInput:
        sei

skipInput:


;Send data to wired connection if controlBit is set
        sbrs    controlBits, 0
        rjmp    skipOutput
        sbic    PINB, 3
        rjmp    skipOutput

        cli

        in      temp01, PORTB
        andi    temp01, 0b11110001
        out     PORTB, temp01

        in      temp01, DDRB
        ori     temp01, 0b00001110
        out     DDRB, temp01

        in      temp01, PORTB
        ori     temp01, 0b00001110
        out     PORTB, temp01

        clr     temp01
waitOutput01:
        inc     temp01
        cpi     temp01, 255
        brne    waitOutput01

        ldi     temp02, MODULE_ID_TO_SEND
        rcall   writeOutput

        mov     temp02, IrCommandNr
        rcall   writeOutput

        in      temp01, PORTB
        andi    temp01, 0b11110001
        out     PORTB, temp01

        andi    controlBits, 0b11111110

        in      temp01, DDRB
        andi    temp01, 0b11110001
        out     DDRB, temp01

        in      temp01, PORTB
        andi    temp01, 0b11110001
        out     PORTB, temp01

        sei

skipOutput:




;Switch IR-LED to send module ID and to receive commands

        cpi     irLEDcounter, 1    ;Reset IR communication if nothing happened until counter reached 0
        brsh    irLED01
        andi    controlBits, 0b11110001
        clr     moduleIRid
        cbi     PORTB, 5    ;Turn off IR LEDs
        cbi     PORTB, 0    ;Turn off green signal LEDs
        cpi     IRcommand, 5
        brne    irLEDcommand01
        cbi     PORTD, 0    ;Turn table clockwise
        sbi     PORTD, 1
        ldi     MotorRun, 100
        inc     IrCommandNr
irLEDcommand01:
        cpi     IRcommand, 10
        brne    irLEDcommand02
        sbi     PORTD, 0    ;Turn table anticlockwise
        cbi     PORTD, 1
        ldi     MotorRun, 100
        inc     IrCommandNr
irLEDcommand02:
        cpi     IRcommand, 15
        brne    irLEDcommand03
        sbi     PORTC, 0 ;Turn light on
        ldi     lightOn01, 255
        ldi     lightOn02, 10
        inc     IrCommandNr
irLEDcommand03:
        cpi     IRcommand, 20
        brne    irLEDcommand04
        cbi     PORTC, 0 ;Turn light off
        inc     IrCommandNr
irLEDcommand04:

        clr     IRcommand


irLED01:

        sbrc    controlBits, 1
        rjmp    irLED04

        andi    controlBits, 0b11110111 ;Store current state of IR sensor
        sbis    PINB, 4
        ori     controlBits, 0b00001000 ;Controlbit set if sensor illuminated

        ;Reset irLEDcounter to initial value if pulse is not long enough
        sbrs    controlBits, 3  ;IR-LED is turned off, sensor is illuminated
        rjmp    irLED02
        sbis    PINB, 0
        sbi     PORTB, 0    ;Turn on green signal LEDs
        sbrc    controlBits, 2
        rjmp    irLED02
        ori     controlBits, 0b00000100
        ldi     irLEDcounter,85
irLED02:

        sbrc    controlBits, 3  ;IR-LED is turned off, sensor is NOT illuminated
        rjmp    irLED03
        sbrs    controlBits, 2
        rjmp    irLED03
        andi    controlBits, 0b11111011
        ldi     irLEDcounter,85
irLED03:

        sbrs    controlBits, 3  ;IR-LED is turned off, sensor is illuminated long enough to start communication
        rjmp    irLED04
        cpi     irLEDcounter, 7
        brsh    irLED04
        ori     controlBits, 0b00000010
        andi    controlBits, 0b11110011
        ldi     irLEDcounter,200
irLED04:

        ;Send IR ID afer a long pulse from rover
        sbrs    controlBits, 1
        rjmp    irLEDend

        sbic    PINB, 4 ;Wait until sensor is not illuminated after long start pulse
        ori     controlBits, 0b00001000 ;Start sending IR-ID

        sbrs    controlBits, 3
        rjmp    irLEDend

        cpi     moduleIRid, MODULE_ID_TO_SEND
        brsh    irLED06

        cpi     irLEDcounter, 3  ;Short ON, long OFF time
        brsh    irLED05
        sbrc    controlBits, 2
        rjmp    irLED05
        ori     controlBits, 0b00000100
        sbi     PORTB, 5       ;Turn on IR LEDs
        ldi     irLEDcounter, 15
irLED05:

        cpi     irLEDcounter, 9  ;Short ON, long OFF time
        brsh    irLED06
        sbrs    controlBits, 2
        rjmp    irLED06
        andi    controlBits, 0b11111011
        cbi     PORTB, 5       ;Turn off IR LEDs
        ldi     irLEDcounter, 30
        inc     moduleIRid
        cpi     moduleIRid, MODULE_ID_TO_SEND
        brlo    irLED06
        ldi     irLEDcounter, 255  ;Wait some longer for commands after IR ID was sent
        ori     controlBits, 0b00000100
irLED06:


        ;Get instructions via IR-interface after IrID was sent completely
        cpi     moduleIRid, MODULE_ID_TO_SEND
        brlo    irLEDend

        sbic    PINB, 4
        rjmp    irLEDget01
        sbrc    controlBits, 2
        rjmp    irLEDget01
        inc     IRcommand
        ori     controlBits, 0b00000100
        ldi     irLEDcounter, 255
irLEDget01:

        sbis    PINB, 4
        rjmp    irLEDget02
        sbrs    controlBits, 2
        rjmp    irLEDget02
        andi    controlBits, 0b11111011
        ldi     irLEDcounter, 255
irLEDget02:

irLEDend:

        cpi     MotorRun, 0
        brne    SkipMotorStop
        cbi     PORTD, 0
        cbi     PORTD, 1
SkipMotorStop:

        cpi     lightOn01, 0
        brne    SkipLightOff
        cbi     PORTC, 0 ;Turn light off
SkipLightOff:


        rjmp    loop


;++++++++++++++++++++++++++++++++++ Write Output values +++++++++++++++++++++++++++++++++++

writeOutput:

        in       temp01, PORTB
        andi     temp01, 0b11111011
        sbrc     temp02, 0
        out      PORTB, temp01

        in       temp01, PORTB
        andi     temp01, 0b11111101
        sbrs     temp02, 0
        out      PORTB, temp01
        in       temp01, PORTB
        andi     temp01, 0b11111001
        sbrs     temp02, 0
        out      PORTB, temp01

        clr     temp01
writeOutputWait01:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait01

        in      temp01, PORTB
        ori     temp01, 0b00001110
        out     PORTB, temp01

        clr     temp01
writeOutputWait01a:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait01a


        in       temp01, PORTB
        andi     temp01, 0b11111011
        sbrc     temp02, 1
        out      PORTB, temp01

        in       temp01, PORTB
        andi     temp01, 0b11111101
        sbrs     temp02, 1
        out      PORTB, temp01
        in       temp01, PORTB
        andi     temp01, 0b11111001
        sbrs     temp02, 1
        out      PORTB, temp01

        clr     temp01
writeOutputWait02:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait02

        in      temp01, PORTB
        ori     temp01, 0b00001110
        out     PORTB, temp01

        clr     temp01
writeOutputWait02a:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait02a


        in       temp01, PORTB
        andi     temp01, 0b11111011
        sbrc     temp02, 2
        out      PORTB, temp01

        in       temp01, PORTB
        andi     temp01, 0b11111101
        sbrs     temp02, 2
        out      PORTB, temp01
        in       temp01, PORTB
        andi     temp01, 0b11111001
        sbrs     temp02, 2
        out      PORTB, temp01

        clr     temp01
writeOutputWait03:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait03

        in      temp01, PORTB
        ori     temp01, 0b00001110
        out     PORTB, temp01

        clr     temp01
writeOutputWait03a:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait03a


        in       temp01, PORTB
        andi     temp01, 0b11111011
        sbrc     temp02, 3
        out      PORTB, temp01

        in       temp01, PORTB
        andi     temp01, 0b11111101
        sbrs     temp02, 3
        out      PORTB, temp01
        in       temp01, PORTB
        andi     temp01, 0b11111001
        sbrs     temp02, 3
        out      PORTB, temp01

        clr     temp01
writeOutputWait04:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait04

        in      temp01, PORTB
        ori     temp01, 0b00001110
        out     PORTB, temp01

        clr     temp01
writeOutputWait04a:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait04a


        in       temp01, PORTB
        andi     temp01, 0b11111011
        sbrc     temp02, 4
        out      PORTB, temp01

        in       temp01, PORTB
        andi     temp01, 0b11111101
        sbrs     temp02, 4
        out      PORTB, temp01
        in       temp01, PORTB
        andi     temp01, 0b11111001
        sbrs     temp02, 4
        out      PORTB, temp01

        clr     temp01
writeOutputWait05:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait05

        in      temp01, PORTB
        ori     temp01, 0b00001110
        out     PORTB, temp01

        clr     temp01
writeOutputWait05a:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait05a


        in       temp01, PORTB
        andi     temp01, 0b11111011
        sbrc     temp02, 5
        out      PORTB, temp01

        in       temp01, PORTB
        andi     temp01, 0b11111101
        sbrs     temp02, 5
        out      PORTB, temp01
        in       temp01, PORTB
        andi     temp01, 0b11111001
        sbrs     temp02, 5
        out      PORTB, temp01

        clr     temp01
writeOutputWait06:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait06

        in      temp01, PORTB
        ori     temp01, 0b00001110
        out     PORTB, temp01

        clr     temp01
writeOutputWait06a:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait06a


        in       temp01, PORTB
        andi     temp01, 0b11111011
        sbrc     temp02, 6
        out      PORTB, temp01

        in       temp01, PORTB
        andi     temp01, 0b11111101
        sbrs     temp02, 6
        out      PORTB, temp01
        in       temp01, PORTB
        andi     temp01, 0b11111001
        sbrs     temp02, 6
        out      PORTB, temp01

        clr     temp01
writeOutputWait07:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait07

        in      temp01, PORTB
        ori     temp01, 0b00001110
        out     PORTB, temp01

        clr     temp01
writeOutputWait07a:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait07a


        in       temp01, PORTB
        andi     temp01, 0b11111011
        sbrc     temp02, 7
        out      PORTB, temp01

        in       temp01, PORTB
        andi     temp01, 0b11111101
        sbrs     temp02, 7
        out      PORTB, temp01
        in       temp01, PORTB
        andi     temp01, 0b11111001
        sbrs     temp02, 7
        out      PORTB, temp01

        clr     temp01
writeOutputWait08:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait08

        in      temp01, PORTB
        ori     temp01, 0b00001110
        out     PORTB, temp01

        clr     temp01
writeOutputWait08a:
        inc     temp01
        cpi     temp01, 255
        brne    writeOutputWait08a

        ret

;++++++++++++++++++++++++++++++++++ Read Input values +++++++++++++++++++++++++++++++++++++

readInput:

inputWait01:
        sbis    PINB, 3
        rjmp    abortInput
        sbic    PINB, 2
        rjmp    inputWait01
        sbic    PINB, 1
        ori     temp01, 0b00000001
input01:
        sbis    PINB, 3
        rjmp    abortInput
        sbis    PINB, 2
        rjmp    input01

inputWait02:
        sbis    PINB, 3
        rjmp    abortInput
        sbic    PINB, 2
        rjmp    inputWait02
        sbic    PINB, 1
        ori     temp01, 0b00000010
input02:
        sbis    PINB, 3
        rjmp    abortInput
        sbis    PINB, 2
        rjmp    input02

inputWait03:
        sbis    PINB, 3
        rjmp    abortInput
        sbic    PINB, 2
        rjmp    inputWait03
        sbic    PINB, 1
        ori     temp01, 0b00000100
input03:
        sbis    PINB, 3
        rjmp    abortInput
        sbis    PINB, 2
        rjmp    input03

inputWait04:
        sbis    PINB, 3
        rjmp    abortInput
        sbic    PINB, 2
        rjmp    inputWait04
        sbic    PINB, 1
        ori     temp01, 0b00001000
input04:
        sbis    PINB, 3
        rjmp    abortInput
        sbis    PINB, 2
        rjmp    input04

inputWait05:
        sbis    PINB, 3
        rjmp    abortInput
        sbic    PINB, 2
        rjmp    inputWait05
        sbic    PINB, 1
        ori     temp01, 0b00010000
input05:
        sbis    PINB, 3
        rjmp    abortInput
        sbis    PINB, 2
        rjmp    input05

inputWait06:
        sbis    PINB, 3
        rjmp    abortInput
        sbic    PINB, 2
        rjmp    inputWait06
        sbic    PINB, 1
        ori     temp01, 0b00100000
input06:
        sbis    PINB, 3
        rjmp    abortInput
        sbis    PINB, 2
        rjmp    input06

inputWait07:
        sbis    PINB, 3
        rjmp    abortInput
        sbic    PINB, 2
        rjmp    inputWait07
        sbic    PINB, 1
        ori     temp01, 0b01000000
input07:
        sbis    PINB, 3
        rjmp    abortInput
        sbis    PINB, 2
        rjmp    input07

inputWait08:
        sbis    PINB, 3
        rjmp    abortInput
        sbic    PINB, 2
        rjmp    inputWait08
        sbic    PINB, 1
        ori     temp01, 0b10000000
input08:
        sbis    PINB, 3
        rjmp    abortInput
        sbis    PINB, 2
        rjmp    input08

        ret

abortInput:
        ldi     temp02, 0b11111111
        ret

;++++++++++++++++++++++++++++++++++ Timer 2 interrupt +++++++++++++++++++++++++++++++++++++++

timer2_compare:                      ;Timer 2 compare handler

        push    temp01                ;save temp01
        in      temp01, SREG          ;save status register

        cpi     irLEDcounter, 0
        breq    timer2_compare01
        dec     irLEDcounter
timer2_compare01:

        out     SREG, temp01          ;restore status register
        pop     temp01                ;restore temp01
        reti

;++++++++++++++++++++++++++++++++++ Timer 0 interrupt +++++++++++++++++++++++++++++++++++++++

timer0_overflow:                      ;Timer 0 Overflow Handler

        push    temp01                ;save temp01
        in      temp01, SREG          ;save status register

        cpi     MotorRun, 0
        breq    timer0_overflow01
        dec     MotorRun
timer0_overflow01:

        cpi     lightOn01, 0
        breq    timer0_overflow02
        dec     lightOn01
timer0_overflow02:

        cpi     lightOn01, 1
        brsh    timer0_overflow03
        cpi     lightOn02, 0
        breq    timer0_overflow03
        dec     lightOn02
        ldi     lightOn01, 255
timer0_overflow03:


        out     SREG, temp01          ;restore status register
        pop     temp01                ;restore temp01
        reti
